@@ -16,7 +16,7 @@ from . import backends
 from .. import build
 from .. import dependencies
 from .. import mesonlib
-import uuid, os, sys
+import uuid, os
 
 from ..mesonlib import MesonException
 
@@ -565,9 +565,7 @@ class XCodeBackend(backends.Backend):
         self.write_line(');')
         self.write_line('runOnlyForDeploymentPostprocessing = 0;')
         self.write_line('shellPath = /bin/sh;')
-        script_root = self.environment.get_script_dir()
-        test_script = os.path.join(script_root, 'meson_test.py')
-        cmd = mesonlib.python_command + [test_script, test_data, '--wd', self.environment.get_build_dir()]
+        cmd = mesonlib.meson_command + ['test', test_data, '-C', self.environment.get_build_dir()]
         cmdstr = ' '.join(["'%s'" % i for i in cmd])
         self.write_line('shellScript = "%s";' % cmdstr)
         self.write_line('showEnvVarsInLog = 0;')
@@ -708,7 +706,7 @@ class XCodeBackend(backends.Backend):
                 if isinstance(target, build.SharedLibrary):
                     ldargs = ['-dynamiclib', '-Wl,-headerpad_max_install_names'] + dep_libs
                     install_path = os.path.join(self.environment.get_build_dir(), target.subdir, buildtype)
-                    dylib_version = target.version
+                    dylib_version = target.soversion
                 else:
                     ldargs = dep_libs
                     install_path = ''
