@@ -63,6 +63,7 @@ from .base import (DependencyException, ExternalDependency)
 # Win   / Cygwin:   libboost_<module>.dll.a                                 (location = /usr/lib)
 #                   libboost_<module>.a
 #                   cygboost_<module>_1_64.dll                              (location = /usr/bin)
+# Win   / VS:       boost_<module>-vc<ver>-mt[-gd]-<arch>-1_67.dll          (location = C:/local/boost_1_67_0)
 # Mac   / homebrew: libboost_<module>.dylib + libboost_<module>-mt.dylib    (location = /usr/local/lib)
 # Mac   / macports: libboost_<module>.dylib + libboost_<module>-mt.dylib    (location = /opt/local/lib)
 #
@@ -97,8 +98,7 @@ class BoostDependency(ExternalDependency):
     def __init__(self, environment, kwargs):
         super().__init__('boost', environment, 'cpp', kwargs)
         self.need_static_link = ['boost_exception', 'boost_test_exec_monitor']
-        # FIXME: is this the right way to find the build type?
-        self.is_debug = environment.cmd_line_options.buildtype.startswith('debug')
+        self.is_debug = environment.coredata.get_builtin_option('buildtype').startswith('debug')
         threading = kwargs.get("threading", "multi")
         self.is_multithreading = threading == "multi"
 
@@ -186,7 +186,7 @@ class BoostDependency(ExternalDependency):
 
     def detect_nix_roots(self):
         return [os.path.abspath(os.path.join(x, '..'))
-                for x in self.compiler.get_default_include_dirs()]
+                for x in self.clib_compiler.get_default_include_dirs()]
 
     def detect_win_roots(self):
         res = []
@@ -244,8 +244,8 @@ class BoostDependency(ExternalDependency):
         # and http://stackoverflow.com/questions/37218953/isystem-on-a-system-include-directory-causes-errors
         # for more details
 
-        if include_dir and include_dir not in self.compiler.get_default_include_dirs():
-            args.append("".join(self.compiler.get_include_args(include_dir, True)))
+        if include_dir and include_dir not in self.clib_compiler.get_default_include_dirs():
+            args.append("".join(self.clib_compiler.get_include_args(include_dir, True)))
         return args
 
     def get_requested(self, kwargs):
@@ -257,7 +257,7 @@ class BoostDependency(ExternalDependency):
 
     def detect_headers_and_version(self):
         try:
-            version = self.compiler.get_define('BOOST_LIB_VERSION', '#include <boost/version.hpp>', self.env, self.get_compile_args(), [])
+            version = self.clib_compiler.get_define('BOOST_LIB_VERSION', '#include <boost/version.hpp>', self.env, self.get_compile_args(), [])
         except mesonlib.EnvironmentException:
             return
         except TypeError:
@@ -270,7 +270,6 @@ class BoostDependency(ExternalDependency):
 
     def detect_lib_modules(self):
         self.lib_modules = {}
-
         # 1. Try to find modules using compiler.find_library( )
         if self.find_libraries_with_abi_tags(self.abi_tags()):
             pass
@@ -329,8 +328,22 @@ class BoostDependency(ExternalDependency):
     def debug_tag(self):
         return '-gd' if self.is_debug else ''
 
+    def arch_tag(self):
+        # currently only applies to windows msvc installed binaries
+        if self.env.detect_cpp_compiler(self.want_cross).get_id() != 'msvc':
+            return ''
+        # pre-compiled binaries only added arch tag for versions > 1.64
+        if float(self.version) < 1.65:
+            return ''
+        arch = detect_cpu_family(self.env.coredata.compilers)
+        if arch == 'x86':
+            return '-x32'
+        elif arch == 'x86_64':
+            return '-x64'
+        return ''
+
     def versioned_abi_tag(self):
-        return self.compiler_tag() + self.threading_tag() + self.debug_tag() + self.version_tag()
+        return self.compiler_tag() + self.threading_tag() + self.debug_tag() + self.arch_tag() + self.version_tag()
 
     # FIXME - how to handle different distributions, e.g. for Mac? Currently we handle homebrew and macports, but not fink.
     def abi_tags(self):
@@ -362,7 +375,7 @@ class BoostDependency(ExternalDependency):
         for module in self.requested_modules:
             libname = 'boost_' + module + tag
 
-            args = self.compiler.find_library(libname, self.env, self.extra_lib_dirs())
+            args = self.clib_compiler.find_library(libname, self.env, self.extra_lib_dirs())
             if args is None:
                 mlog.debug("Couldn\'t find library '{}' for boost module '{}'  (ABI tag = '{}')".format(libname, module, tag))
                 all_found = False
@@ -440,7 +453,7 @@ class BoostDependency(ExternalDependency):
         if self.libdir:
             libdirs = [self.libdir]
         elif self.boost_root is None:
-            libdirs = mesonlib.get_library_dirs()
+            libdirs = mesonlib.get_library_dirs(self.env)
         else:
             libdirs = [os.path.join(self.boost_root, 'lib')]
         for libdir in libdirs:
@@ -474,10 +487,10 @@ class BoostDependency(ExternalDependency):
             return [os.path.join(self.boost_root, 'lib')]
         return []
 
-    def get_link_args(self):
+    def get_link_args(self, **kwargs):
         args = []
-        for dir in self.extra_lib_dirs():
-            args += self.compiler.get_linker_search_args(dir)
+        for d in self.extra_lib_dirs():
+            args += self.clib_compiler.get_linker_search_args(d)
         for lib in self.requested_modules:
             args += self.lib_modules['boost_' + lib]
         return args
