@@ -172,6 +172,9 @@ class FeatureCheckBase:
 
     @staticmethod
     def get_target_version(subproject):
+        # Don't do any checks if project() has not been parsed yet
+        if subproject not in mesonlib.project_meson_versions:
+            return ''
         return mesonlib.project_meson_versions[subproject]
 
     def use(self, subproject):
@@ -225,7 +228,7 @@ class FeatureNew(FeatureCheckBase):
 
     @staticmethod
     def get_warning_str_prefix(tv):
-        return 'Project specifies a minimum meson_version \'{}\' which conflicts with:'.format(tv)
+        return 'Project specifies a minimum meson_version \'{}\' but uses features which were added in newer versions:'.format(tv)
 
     def log_usage_warning(self, tv):
         mlog.warning('Project targetting \'{}\' but tried to use feature introduced '
