@@ -1,5 +1,4 @@
 # Copyright 2017 The Meson development team
-from pyclbr import Function
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,8 +14,10 @@ from pyclbr import Function
 
 """Code that creates simple startup projects."""
 
-import os, sys, argparse, re
+import os, sys, argparse, re, shutil, subprocess
 from glob import glob
+from mesonbuild import mesonlib
+from mesonbuild.environment import detect_ninja
 
 lib_h_template = '''#pragma once
 #if defined _WIN32 || defined __CYGWIN__
@@ -107,7 +108,7 @@ pkg_mod.generate(
 )
 '''
 
-hello_c_template  = '''#include <stdio.h>
+hello_c_template = '''#include <stdio.h>
 
 #define PROJECT_NAME "{project_name}"
 
@@ -123,16 +124,15 @@ int main(int argc, char **argv) {{
 
 hello_c_meson_template = '''project('{project_name}', 'c',
   version : '{version}',
-  default_options : ['warning_level=3',
-                     'cpp_std=c++14'])
+  default_options : ['warning_level=3'])
 
 exe = executable('{exe_name}', '{source_name}',
   install : true)
-  
+
 test('basic', exe)
 '''
 
-hello_cpp_template  = '''#include <iostream>
+hello_cpp_template = '''#include <iostream>
 
 #define PROJECT_NAME "{project_name}"
 
@@ -148,11 +148,12 @@ int main(int argc, char **argv) {{
 
 hello_cpp_meson_template = '''project('{project_name}', 'cpp',
   version : '{version}',
-  default_options : ['warning_level=3'])
+  default_options : ['warning_level=3',
+                     'cpp_std=c++14'])
 
 exe = executable('{exe_name}', '{source_name}',
   install : true)
-  
+
 test('basic', exe)
 '''
 
@@ -178,9 +179,9 @@ class {utoken}_PUBLIC {class_name} {{
 public:
   {class_name}();
   int get_number() const;
-  
+
 private:
-  
+
   int number;
 
 }};
@@ -270,7 +271,6 @@ ninja -C builddir
 
 def create_exe_c_sample(project_name, project_version):
     lowercase_token = re.sub(r'[^a-z0-9]', '_', project_name.lower())
-    uppercase_token = lowercase_token.upper()
     source_name = lowercase_token + '.c'
     open(source_name, 'w').write(hello_c_template.format(project_name=project_name))
     open('meson.build', 'w').write(hello_c_meson_template.format(project_name=project_name,
@@ -291,7 +291,7 @@ def create_lib_c_sample(project_name, version):
               'function_name': function_name,
               'header_file': lib_h_name,
               'source_file': lib_c_name,
-              'test_source_file': test_c_name, 
+              'test_source_file': test_c_name,
               'test_exe_name': lowercase_token,
               'project_name': project_name,
               'lib_name': lowercase_token,
@@ -305,13 +305,12 @@ def create_lib_c_sample(project_name, version):
 
 def create_exe_cpp_sample(project_name, project_version):
     lowercase_token = re.sub(r'[^a-z0-9]', '_', project_name.lower())
-    uppercase_token = lowercase_token.upper()
     source_name = lowercase_token + '.cpp'
     open(source_name, 'w').write(hello_cpp_template.format(project_name=project_name))
     open('meson.build', 'w').write(hello_cpp_meson_template.format(project_name=project_name,
-                                                                 exe_name=lowercase_token,
-                                                                 source_name=source_name,
-                                                                 version=project_version))
+                                                                   exe_name=lowercase_token,
+                                                                   source_name=source_name,
+                                                                   version=project_version))
 
 def create_lib_cpp_sample(project_name, version):
     lowercase_token = re.sub(r'[^a-z0-9]', '_', project_name.lower())
@@ -328,7 +327,7 @@ def create_lib_cpp_sample(project_name, version):
               'namespace': namespace,
               'header_file': lib_h_name,
               'source_file': lib_c_name,
-              'test_source_file': test_c_name, 
+              'test_source_file': test_c_name,
               'test_exe_name': lowercase_token,
               'project_name': project_name,
               'lib_name': lowercase_token,
@@ -359,15 +358,125 @@ def create_sample(options):
         raise RuntimeError('Unreachable code')
     print(info_message)
 
+def autodetect_options(options, sample=False):
+    if not options.name:
+        options.name = os.path.basename(os.getcwd())
+        if not re.match('[a-zA-Z_][a-zA-Z0-9]*', options.name) and sample:
+            print('Name of current directory "{}" is not usable as a sample project name.\n'
+                  'Specify a project name with --name.'.format(options.name))
+            sys.exit(1)
+        print('Using "{}" (name of current directory) as project name.'
+              .format(options.name))
+    if not options.executable:
+        options.executable = options.name
+        print('Using "{}" (project name) as name of executable to build.'
+              .format(options.executable))
+    if sample:
+        # The rest of the autodetection is not applicable to generating sample projects.
+        return
+    if not options.srcfiles:
+        srcfiles = []
+        for f in os.listdir():
+            if f.endswith('.cc') or f.endswith('.cpp') or f.endswith('.c'):
+                srcfiles.append(f)
+        if not srcfiles:
+            print("No recognizable source files found.\n"
+                  "Run me in an empty directory to create a sample project.")
+            sys.exit(1)
+        options.srcfiles = srcfiles
+        print("Detected source files: " + ' '.join(srcfiles))
+    if not options.language:
+        for f in options.srcfiles:
+            if f.endswith('.cc') or f.endswith('.cpp'):
+                options.language = 'cpp'
+                break
+            if f.endswith('.c'):
+                options.language = 'c'
+                break
+        if not options.language:
+            print("Can't autodetect language, please specify it with -l.")
+            sys.exit(1)
+        print("Detected language: " + options.language)
+
+meson_executable_template = '''project('{project_name}', '{language}',
+  version : '{version}',
+  default_options : [{default_options}])
+
+executable('{executable}',
+           {sourcespec},{depspec}
+           install : true)
+'''
+
+def create_meson_build(options):
+    if options.type != 'executable':
+        print('\nGenerating a meson.build file from existing sources is\n'
+              'supported only for project type "executable".\n'
+              'Run me in an empty directory to create a sample project.')
+        sys.exit(1)
+    default_options = ['warning_level=3']
+    if options.language == 'cpp':
+        # This shows how to set this very common option.
+        default_options += ['cpp_std=c++14']
+    # If we get a meson.build autoformatter one day, this code could
+    # be simplified quite a bit.
+    formatted_default_options = ', '.join("'{}'".format(x) for x in default_options)
+    sourcespec = ',\n           '.join("'{}'".format(x) for x in options.srcfiles)
+    depspec = ''
+    if options.deps:
+        depspec = '\n           dependencies : [\n              '
+        depspec += ',\n              '.join("dependency('{}')".format(x)
+                                            for x in options.deps.split(','))
+        depspec += '],'
+    content = meson_executable_template.format(project_name=options.name,
+                                               language=options.language,
+                                               version=options.version,
+                                               executable=options.executable,
+                                               sourcespec=sourcespec,
+                                               depspec=depspec,
+                                               default_options=formatted_default_options)
+    open('meson.build', 'w').write(content)
+    print('Generated meson.build file:\n\n' + content)
+
 def run(args):
     parser = argparse.ArgumentParser(prog='meson')
-    parser.add_argument('--name', default = 'mesonsample')
+    parser.add_argument("srcfiles", metavar="sourcefile", nargs="*",
+                        help="source files. default: all recognized files in current directory")
+    parser.add_argument("-n", "--name", help="project name. default: name of current directory")
+    parser.add_argument("-e", "--executable", help="executable name. default: project name")
+    parser.add_argument("-d", "--deps", help="dependencies, comma-separated")
+    parser.add_argument("-l", "--language", choices=['c', 'cpp'],
+                        help="project language. default: autodetected based on source files")
+    parser.add_argument("-b", "--build", help="build after generation", action='store_true')
+    parser.add_argument("--builddir", help="directory for build", default='build')
+    parser.add_argument("-f", "--force", action="store_true",
+                        help="force overwrite of existing files and directories.")
     parser.add_argument('--type', default='executable',
                         choices=['executable', 'library'])
-    parser.add_argument('--language', default='c', choices=['c', 'cpp'])
-    parser.add_argument('--version', default='1.0')
+    parser.add_argument('--version', default='0.1')
     options = parser.parse_args(args)
-    if len(glob('*')) != 0:
-        sys.exit('This command must be run in an empty directory.')
-    create_sample(options)
+    if len(glob('*')) == 0:
+        autodetect_options(options, sample=True)
+        if not options.language:
+            print('Defaulting to generating a C language project.')
+            options.language = 'c'
+        create_sample(options)
+    else:
+        autodetect_options(options)
+        if os.path.isfile('meson.build') and not options.force:
+            print('meson.build already exists. Use --force to overwrite.')
+            sys.exit(1)
+        create_meson_build(options)
+    if options.build:
+        if os.path.isdir(options.builddir) and options.force:
+            print('Build directory already exists, deleting it.')
+            shutil.rmtree(options.builddir)
+        print('Building...')
+        cmd = mesonlib.meson_command + [options.builddir]
+        err = subprocess.call(cmd)
+        if err:
+            sys.exit(1)
+        cmd = [detect_ninja(), '-C', options.builddir]
+        err = subprocess.call(cmd)
+        if err:
+            sys.exit(1)
     return 0
