@@ -20,7 +20,16 @@ from .scripts import depfixer
 from .scripts import destdir_join
 from .mesonlib import is_windows, Popen_safe
 from .mtest import rebuild_all
-from __main__ import __file__ as main_file
+try:
+    from __main__ import __file__ as main_file
+except ImportError:
+    # Happens when running as meson.exe which is native Windows.
+    # This is only used for pkexec which is not, so this is fine.
+    main_file = None
+
+symlink_warning = '''Warning: trying to copy a symlink that points to a file. This will copy the file,
+but this will be changed in a future version of Meson to copy the symlink as is. Please update your
+build definitions so that it will not break when the change happens.'''
 
 selinux_updates = []
 
@@ -64,9 +73,9 @@ class DirMaker:
         for d in self.dirs:
             append_to_log(self.lf, d)
 
-def is_executable(path):
+def is_executable(path, follow_symlinks=False):
     '''Checks whether any of the "x" bits are set in the source file mode.'''
-    return bool(os.stat(path).st_mode & 0o111)
+    return bool(os.stat(path, follow_symlinks=follow_symlinks).st_mode & 0o111)
 
 def append_to_log(lf, line):
     lf.write(line)
@@ -74,13 +83,38 @@ def append_to_log(lf, line):
         lf.write('\n')
     lf.flush()
 
+def set_chown(path, user=None, group=None, dir_fd=None, follow_symlinks=True):
+    # shutil.chown will call os.chown without passing all the parameters
+    # and particularly follow_symlinks, thus we replace it temporary
+    # with a lambda with all the parameters so that follow_symlinks will
+    # be actually passed properly.
+    # Not nice, but better than actually rewriting shutil.chown until
+    # this python bug is fixed: https://bugs.python.org/issue18108
+    real_os_chown = os.chown
+    try:
+        os.chown = lambda p, u, g: real_os_chown(p, u, g,
+                                                 dir_fd=dir_fd,
+                                                 follow_symlinks=follow_symlinks)
+        shutil.chown(path, user, group)
+    except:
+        raise
+    finally:
+        os.chown = real_os_chown
+
+def set_chmod(path, mode, dir_fd=None, follow_symlinks=True):
+    try:
+        os.chmod(path, mode, dir_fd=dir_fd, follow_symlinks=follow_symlinks)
+    except (NotImplementedError, OSError, SystemError) as e:
+        if not os.path.islink(path):
+            os.chmod(path, mode, dir_fd=dir_fd)
+
 def sanitize_permissions(path, umask):
     if umask is None:
         return
-    new_perms = 0o777 if is_executable(path) else 0o666
+    new_perms = 0o777 if is_executable(path, follow_symlinks=False) else 0o666
     new_perms &= ~umask
     try:
-        os.chmod(path, new_perms)
+        set_chmod(path, new_perms, follow_symlinks=False)
     except PermissionError as e:
         msg = '{!r}: Unable to set permissions {!r}: {}, ignoring...'
         print(msg.format(path, new_perms, e.strerror))
@@ -93,7 +127,7 @@ def set_mode(path, mode, default_umask):
     # No chown() on Windows, and must set one of owner/group
     if not is_windows() and (mode.owner or mode.group) is not None:
         try:
-            shutil.chown(path, mode.owner, mode.group)
+            set_chown(path, mode.owner, mode.group, follow_symlinks=False)
         except PermissionError as e:
             msg = '{!r}: Unable to set owner {!r} and group {!r}: {}, ignoring...'
             print(msg.format(path, mode.owner, mode.group, e.strerror))
@@ -111,7 +145,7 @@ def set_mode(path, mode, default_umask):
     # NOTE: On Windows you can set read/write perms; the rest are ignored
     if mode.perms_s is not None:
         try:
-            os.chmod(path, mode.perms)
+            set_chmod(path, mode.perms, follow_symlinks=False)
         except PermissionError as e:
             msg = '{!r}: Unable to set permissions {!r}: {}, ignoring...'
             print(msg.format(path, mode.perms_s, e.strerror))
@@ -186,12 +220,16 @@ class Installer:
     def should_preserve_existing_file(self, from_file, to_file):
         if not self.options.only_changed:
             return False
+        # Always replace danging symlinks
+        if os.path.islink(from_file) and not os.path.isfile(from_file):
+            return False
         from_time = os.stat(from_file).st_mtime
         to_time = os.stat(to_file).st_mtime
         return from_time <= to_time
 
     def do_copyfile(self, from_file, to_file):
-        if not os.path.isfile(from_file):
+        outdir = os.path.split(to_file)[0]
+        if not os.path.isfile(from_file) and not os.path.islink(from_file):
             raise RuntimeError('Tried to install something that isn\'t a file:'
                                '{!r}'.format(from_file))
         # copyfile fails if the target file already exists, so remove it to
@@ -205,11 +243,21 @@ class Installer:
                 append_to_log(self.lf, '# Preserving old file %s\n' % to_file)
                 print('Preserving existing file %s.' % to_file)
                 return False
-            os.unlink(to_file)
-        outdir = os.path.split(to_file)[0]
+            os.remove(to_file)
         print('Installing %s to %s' % (from_file, outdir))
-        shutil.copyfile(from_file, to_file)
-        shutil.copystat(from_file, to_file)
+        if os.path.islink(from_file):
+            if not os.path.exists(from_file):
+                # Dangling symlink. Replicate as is.
+                shutil.copy(from_file, outdir, follow_symlinks=False)
+            else:
+                # Remove this entire branch when changing the behaviour to duplicate
+                # symlinks rather than copying what they point to.
+                print(symlink_warning)
+                shutil.copyfile(from_file, to_file)
+                shutil.copystat(from_file, to_file)
+        else:
+            shutil.copyfile(from_file, to_file)
+            shutil.copystat(from_file, to_file)
         selinux_updates.append(to_file)
         append_to_log(self.lf, to_file)
         return True
@@ -272,7 +320,7 @@ class Installer:
                 if os.path.isdir(abs_dst):
                     print('Tried to copy file %s but a directory of that name already exists.' % abs_dst)
                 if os.path.exists(abs_dst):
-                    os.unlink(abs_dst)
+                    os.remove(abs_dst)
                 parent_dir = os.path.dirname(abs_dst)
                 if not os.path.isdir(parent_dir):
                     os.mkdir(parent_dir)
@@ -306,7 +354,7 @@ class Installer:
                 print('Installation failed due to insufficient permissions.')
                 print('Attempting to use polkit to gain elevated privileges...')
                 os.execlp('pkexec', 'pkexec', sys.executable, main_file, *sys.argv[1:],
-                          os.getcwd())
+                          '-C', os.getcwd())
             else:
                 raise
 
@@ -426,7 +474,7 @@ class Installer:
                 try:
                     symlinkfilename = os.path.join(outdir, alias)
                     try:
-                        os.unlink(symlinkfilename)
+                        os.remove(symlinkfilename)
                     except FileNotFoundError:
                         pass
                     os.symlink(to, symlinkfilename)
