@@ -511,7 +511,7 @@ class GnomeModule(ExtensionModule):
                 sanitize = state.environment.coredata.base_options['b_sanitize'].value
                 cflags += compilers.sanitizer_compile_args(sanitize)
                 if 'address' in sanitize.split(','):
-                    external_ldflags += ['-lasan']
+                    internal_ldflags += ['-lasan']  # This must be first in ldflags
                 # FIXME: Linking directly to libasan is not recommended but g-ir-scanner
                 # does not understand -f LDFLAGS. https://bugzilla.gnome.org/show_bug.cgi?id=783892
                 # ldflags += compilers.sanitizer_link_args(sanitize)
@@ -683,7 +683,7 @@ class GnomeModule(ExtensionModule):
 
         langs = mesonlib.stringlistify(kwargs.pop('languages', []))
         if langs:
-            mlog.log(mlog.red('DEPRECATION:'), '''The "languages" argument of gnome.yelp() is deprecated.
+            mlog.deprecation('''The "languages" argument of gnome.yelp() is deprecated.
 Use a LINGUAS file in the sources directory instead.
 This will become a hard error in the future.''')
 
@@ -831,7 +831,7 @@ This will become a hard error in the future.''')
 
         args += self._unpack_args('--expand-content-files=', 'expand_content_files', kwargs, state)
         args += self._unpack_args('--ignore-headers=', 'ignore_headers', kwargs)
-        args += self._unpack_args('--installdir=', 'install_dir', kwargs, state)
+        args += self._unpack_args('--installdir=', 'install_dir', kwargs)
         args += self._get_build_args(kwargs, state, depends)
         res = [build.RunTarget(targetname, command[0], command[1:] + args, depends, state.subdir, state.subproject)]
         if kwargs.get('install', True):
@@ -1018,7 +1018,7 @@ This will become a hard error in the future.''')
             output = namebase + '-docbook'
             outputs = []
             for f in xml_files:
-                outputs.append('{}-{}'.format(docbook, f))
+                outputs.append('{}-{}'.format(docbook, os.path.basename(str(f))))
 
             if mesonlib.version_compare(glib_version, '>= 2.56.2'):
                 custom_kwargs = {'input': xml_files,
