@@ -19,11 +19,11 @@ from . import ExtensionModule
 
 class Qt5Module(ExtensionModule, QtBaseModule):
 
-    def __init__(self):
+    def __init__(self, interpreter):
         QtBaseModule.__init__(self, qt_version=5)
-        ExtensionModule.__init__(self)
+        ExtensionModule.__init__(self, interpreter)
 
-def initialize():
+def initialize(*args, **kwargs):
     mlog.warning('rcc dependencies will not work reliably until this upstream issue is fixed:',
                  mlog.bold('https://bugreports.qt.io/browse/QTBUG-45460'))
-    return Qt5Module()
+    return Qt5Module(*args, **kwargs)
