@@ -15,12 +15,14 @@
 import os
 
 from .. import mlog
-from .. import mesonlib, dependencies, build
+from .. import mesonlib, build
 from ..mesonlib import MesonException, extract_as_list
 from . import get_include_args
 from . import ModuleReturnValue
 from . import ExtensionModule
-from ..interpreterbase import permittedKwargs
+from ..interpreter import CustomTargetHolder
+from ..interpreterbase import permittedKwargs, FeatureNewKwargs
+from ..dependencies import ExternalProgram
 
 class WindowsModule(ExtensionModule):
 
@@ -30,11 +32,17 @@ class WindowsModule(ExtensionModule):
                 return compilers[l]
         raise MesonException('Resource compilation requires a C or C++ compiler.')
 
-    @permittedKwargs({'args', 'include_directories'})
+    @FeatureNewKwargs('windows.compile_resources', '0.47.0', ['depend_files', 'depends'])
+    @permittedKwargs({'args', 'include_directories', 'depend_files', 'depends'})
     def compile_resources(self, state, args, kwargs):
         comp = self.detect_compiler(state.compilers)
 
         extra_args = mesonlib.stringlistify(kwargs.get('args', []))
+        wrc_depend_files = extract_as_list(kwargs, 'depend_files', pop = True)
+        wrc_depends = extract_as_list(kwargs, 'depends', pop = True)
+        for d in wrc_depends:
+            if isinstance(d, CustomTargetHolder):
+                extra_args += get_include_args([d.outdir_include()])
         inc_dirs = extract_as_list(kwargs, 'include_directories', pop = True)
         for incd in inc_dirs:
             if not isinstance(incd.held_object, (str, build.IncludeDirs)):
@@ -42,7 +50,7 @@ class WindowsModule(ExtensionModule):
         extra_args += get_include_args(inc_dirs)
 
         if comp.id == 'msvc':
-            rescomp = dependencies.ExternalProgram('rc', silent=True)
+            rescomp = ExternalProgram('rc', silent=True)
             res_args = extra_args + ['/nologo', '/fo@OUTPUT@', '@INPUT@']
             suffix = 'res'
         else:
@@ -51,22 +59,23 @@ class WindowsModule(ExtensionModule):
             for arg in extra_args:
                 if ' ' in arg:
                     mlog.warning(m.format(arg))
-            rescomp_name = None
+            rescomp = None
             # FIXME: Does not handle `native: true` executables, see
             # https://github.com/mesonbuild/meson/issues/1531
             if state.environment.is_cross_build():
                 # If cross compiling see if windres has been specified in the
                 # cross file before trying to find it another way.
-                rescomp_name = state.environment.cross_info.config['binaries'].get('windres')
-            if rescomp_name is None:
+                cross_info = state.environment.cross_info
+                rescomp = ExternalProgram.from_cross_info(cross_info, 'windres')
+            if not rescomp or not rescomp.found():
                 # Pick-up env var WINDRES if set. This is often used for
                 # specifying an arch-specific windres.
-                rescomp_name = os.environ.get('WINDRES', 'windres')
-            rescomp = dependencies.ExternalProgram(rescomp_name, silent=True)
+                rescomp = ExternalProgram(os.environ.get('WINDRES', 'windres'), silent=True)
             res_args = extra_args + ['@INPUT@', '@OUTPUT@']
             suffix = 'o'
         if not rescomp.found():
-            raise MesonException('Could not find Windows resource compiler %s.' % ' '.join(rescomp.get_command()))
+            raise MesonException('Could not find Windows resource compiler {!r}'
+                                 ''.format(rescomp.get_path()))
 
         res_targets = []
 
@@ -83,10 +92,14 @@ class WindowsModule(ExtensionModule):
                 'output': '@BASENAME@.' + suffix,
                 'input': [src],
                 'command': [rescomp] + res_args,
+                'depend_files': wrc_depend_files,
+                'depends': wrc_depends,
             }
 
-            if isinstance(src, (str, mesonlib.File)):
-                name = 'file {!r}'.format(str(src))
+            if isinstance(src, str):
+                name = 'file {!r}'.format(os.path.join(state.subdir, src))
+            elif isinstance(src, mesonlib.File):
+                name = 'file {!r}'.format(src.relative_name())
             elif isinstance(src, build.CustomTarget):
                 if len(src.get_outputs()) > 1:
                     raise MesonException('windows.compile_resources does not accept custom targets with more than 1 output.')
@@ -98,11 +111,16 @@ class WindowsModule(ExtensionModule):
             # Path separators are not allowed in target names
             name = name.replace('/', '_').replace('\\', '_')
 
+            # instruct binutils windres to generate a preprocessor depfile
+            if comp.id != 'msvc':
+                res_kwargs['depfile'] = res_kwargs['output'] + '.d'
+                res_kwargs['command'] += ['--preprocessor-arg=-MD', '--preprocessor-arg=-MQ@OUTPUT@', '--preprocessor-arg=-MF@DEPFILE@']
+
             res_targets.append(build.CustomTarget('Windows resource for ' + name, state.subdir, state.subproject, res_kwargs))
 
         add_target(args)
 
         return ModuleReturnValue(res_targets, [res_targets])
 
-def initialize():
-    return WindowsModule()
+def initialize(*args, **kwargs):
+    return WindowsModule(*args, **kwargs)
