@@ -14,7 +14,6 @@
 
 import sys, os
 import pickle, subprocess
-from mesonbuild.mesonlib import meson_command
 
 # This could also be used for XCode.
 
@@ -32,7 +31,7 @@ def need_regen(regeninfo, regen_timestamp):
     Vs2010Backend.touch_regen_timestamp(regeninfo.build_dir)
     return False
 
-def regen(regeninfo, mesonscript, backend):
+def regen(regeninfo, meson_command, backend):
     cmd = meson_command + ['--internal',
                            'regenerate',
                            regeninfo.build_dir,
@@ -48,11 +47,10 @@ def run(args):
         regeninfo = pickle.load(f)
     with open(coredata, 'rb') as f:
         coredata = pickle.load(f)
-    mesonscript = coredata.meson_script_launcher
     backend = coredata.get_builtin_option('backend')
     regen_timestamp = os.stat(dumpfile).st_mtime
     if need_regen(regeninfo, regen_timestamp):
-        regen(regeninfo, mesonscript, backend)
+        regen(regeninfo, coredata.meson_command, backend)
     sys.exit(0)
 
 if __name__ == '__main__':
