@@ -14,10 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from mesonbuild import mesonmain
 import sys
 
 if __name__ == '__main__':
-    print('Warning: This executable is deprecated. Use "meson configure" instead.',
-          file=sys.stderr)
-    sys.exit(mesonmain.run(['configure'] + sys.argv[1:]))
+    sys.exit('Error: This executable is no more. Use "meson configure" instead.')
