@@ -18,6 +18,7 @@ import os
 import tempfile
 import unittest
 import subprocess
+import zipapp
 from pathlib import Path
 
 from mesonbuild.mesonlib import windows_proof_rmtree, python_command, is_windows
@@ -171,7 +172,7 @@ class CommandTests(unittest.TestCase):
         builddir = str(self.tmpdir / 'build4')
         (bindir / 'meson').rename(bindir / 'meson.real')
         wrapper = (bindir / 'meson')
-        with open(wrapper, 'w') as f:
+        with open(str(wrapper), 'w') as f:
             f.write('#!/bin/sh\n\nmeson.real "$@"')
         wrapper.chmod(0o755)
         meson_setup = [str(wrapper), 'setup']
@@ -182,5 +183,13 @@ class CommandTests(unittest.TestCase):
     def test_meson_exe_windows(self):
         raise unittest.SkipTest('NOT IMPLEMENTED')
 
+    def test_meson_zipapp(self):
+        if is_windows():
+            raise unittest.SkipTest('NOT IMPLEMENTED')
+        source = Path(__file__).resolve().parent.as_posix()
+        target = self.tmpdir / 'meson.pyz'
+        zipapp.create_archive(source=source, target=target, interpreter=python_command[0], main=None)
+        self._run([target.as_posix(), '--help'])
+
 if __name__ == '__main__':
     unittest.main(buffer=True)
