@@ -1,4 +1,4 @@
-#include"config2.h"
+#include<config2.h>
 
 int main(int argc, char **argv) {
     return ZERO_RESULT;
